#ifndef MIDI_INTERPRETER_H
#define MIDI_INTERPRETER_H
#include "mfile\MidiFile.h"
#include "timer.h"
#include "base.h"
#include "track.h"



struct Playlist {
	std::vector<std::string> data;
	int pos = 0;
};

class Interpreter : public MidiEventListener {
	private:
		Playlist playlist;
		
		std::string songName = "";
		bool fileSongName = true;
		std::string copyright = "";
		std::string lyrics = "";
		std::string completeLyrics = "";
		bool gotSong = false;
		
		ALLEGRO_EVENT_QUEUE *queue;
		Timer timer;
		
		std::vector<Track> tracks;
		int activeTracks = 0;
		MidiFile file;
		unsigned int tickPos = 0;
		unsigned char channelPrefix = 128;
		int longestTrack = 0;
	public:
		//Anfangstempo: 120 BPM
		Interpreter();
		~Interpreter();
		
		void addListener(MidiEventListener *lis) { Track::addListener(lis);}
		void addEventSource(ALLEGRO_EVENT_SOURCE *source) { al_register_event_source(queue, source); }
		void reactEvent(int trackID, const MidiEvent& event);
		void openFile(const std::string& filename);
		
		void play() { timer.start(); }
		void stop() { timer.stop(); activeTracks = 0;}
		void next() { playlist.pos++; if(playlist.pos >= playlist.data.size()) playlist.pos = playlist.data.size()-1;}
		void prev() { playlist.pos--; if(playlist.pos < 0) playlist.pos = 0;}
		
		
		void reset();
		void check();
		
		bool isFinished() {
			return activeTracks <= 0;
		}
		
		const std::string& getSongName() {
			return songName;
		}
		
		const std::string& getCopyright() {
			return copyright;
		}
		
		const std::string& getLyrics() {
			return lyrics;
		}
		
		const std::string& getFullLyrics() {
			return completeLyrics;
		}
		
		unsigned int getTick() {
			return tickPos;
		}
		
		unsigned int getLength() {
			if(tracks.size() == 0) return 0;
			if(longestTrack >= tracks.size()) return 0;
			return tracks[longestTrack].getLength();
		}
		
		bool hasSong() {
			return gotSong;
		}
		
		int getPlaylistSize() {
			return playlist.data.size();
		}
		
		int getPlaylistPos() {
			return playlist.pos;
		}
		
		const std::string& getPlaylist(int i ) {
			return playlist.data[i];
		}
};

#endif
