#include "program.h"

Interpreter *reader;

void *audioThreadR(ALLEGRO_THREAD *thread, void *arg) {
	try {
		while(!al_get_thread_should_stop(thread)) {
			synth->check();
		}
	} catch(std::string& str) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured (In the Synth:)", str.c_str(), "Oh no!", ALLEGRO_MESSAGEBOX_ERROR);
	} catch(std::exception& e) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured (In the Synth:)", e.what(), "Oh bugger!", ALLEGRO_MESSAGEBOX_ERROR);
	}
}

void *interpreterThreadR(ALLEGRO_THREAD *thread, void *arg) {
	try {
		ALLEGRO_EVENT_SOURCE *eventSource = (ALLEGRO_EVENT_SOURCE*) arg;
		reader->addEventSource(eventSource);
		
	
		reader->addListener(synth);
		while(!al_get_thread_should_stop(thread)) {
			reader->check();
		}
		std::cout << "Finished!" << std::endl;
		reader->stop();
	} catch(std::string& str) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured (In the MIDIManager:)", str.c_str(), "Oh no!", ALLEGRO_MESSAGEBOX_ERROR);
	} catch(std::exception& e) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured (In the MIDIManager:)", e.what(), "Oh bugger!", ALLEGRO_MESSAGEBOX_ERROR);
	}
}

void Program::initAllegro(int w, int h) {
	if (!al_init()) {
		throw(std::string("Can't init example!"));
	}
	
	al_install_keyboard();
	al_install_mouse();
	
	open_log();

	al_init_native_dialog_addon();
	al_init_primitives_addon();
	al_init_font_addon();
	al_init_ttf_addon();
	
	display = al_create_display(w, h);
	if (!display) throw(std::string("Unable to create display\n"));
	al_set_window_title(display, "Expos Midi Player");
	
	queue = al_create_event_queue();
	
	al_register_event_source(queue, al_get_display_event_source(display));
	al_register_event_source(queue, al_get_mouse_event_source());
	
	al_init_user_event_source(&programEventSource);
	
	screenW = w;
	screenH = h;
}

void Program::initGui() {
	gui.menu = al_create_menu();
	gui.file_menu = al_create_menu();
	gui.settings = al_create_menu();
	gui.channels = al_create_menu();
	gui.song_menu = al_create_menu();
	gui.help = al_create_menu();
	
	al_append_menu_item(gui.file_menu, "Open", FILE_OPEN_ID, 0, nullptr, nullptr);
	al_append_menu_item(gui.file_menu, "Add to Playlist", FILE_ADD_ID, 0, nullptr, nullptr);
	al_append_menu_item(gui.file_menu, "Export Lyrics", FILE_EXPORT_LID, 0, nullptr, nullptr);
	al_append_menu_item(gui.file_menu, "&Close", FILE_CLOSE_ID, 0, nullptr, nullptr);
	
	al_append_menu_item(gui.song_menu, "Play", FILE_PLAY_ID, 0, nullptr, nullptr);
	al_append_menu_item(gui.song_menu, "Previous", FILE_PREV_ID, 0, nullptr, nullptr);
	al_append_menu_item(gui.song_menu, "Next", FILE_NEXT_ID, 0, nullptr, nullptr);
	al_append_menu_item(gui.song_menu, "Stop", FILE_STOP_ID, 0, nullptr, nullptr);
	
	al_append_menu_item(gui.channels, "Channel 01", FILE_CHANNEL_ID(0), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 02", FILE_CHANNEL_ID(1), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 03", FILE_CHANNEL_ID(2), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 04", FILE_CHANNEL_ID(3), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 05", FILE_CHANNEL_ID(4), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 06", FILE_CHANNEL_ID(5), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 07", FILE_CHANNEL_ID(6), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 08", FILE_CHANNEL_ID(7), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 09", FILE_CHANNEL_ID(8), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Drums, C10", FILE_CHANNEL_ID(9), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 11", FILE_CHANNEL_ID(10), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 12", FILE_CHANNEL_ID(11), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 13", FILE_CHANNEL_ID(12), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 14", FILE_CHANNEL_ID(13), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 15", FILE_CHANNEL_ID(14), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	al_append_menu_item(gui.channels, "Channel 16", FILE_CHANNEL_ID(15), ALLEGRO_MENU_ITEM_CHECKED, nullptr, nullptr);
	
	al_append_menu_item(gui.settings, "Channels", 0, 0, nullptr, gui.channels);
	
	al_append_menu_item(gui.help, "Info", FILE_INFO_ID, 0, nullptr, nullptr);
	
	al_append_menu_item(gui.menu, "File", 0, 0, nullptr, gui.file_menu);
	al_append_menu_item(gui.menu, "Song", 0, 0, nullptr, gui.song_menu);
	al_append_menu_item(gui.menu, "Settings", 0, 0, nullptr, gui.settings);
	al_append_menu_item(gui.menu, "Help", 0, 0, nullptr, gui.help);
	al_set_display_menu(display, gui.menu);
	
	al_register_event_source(queue, al_get_default_menu_event_source());
	gui.fileDialog = al_create_native_file_dialog(resourcePath.c_str(), "Open File", "*.mid", ALLEGRO_FILECHOOSER_FILE_MUST_EXIST);
	gui.exportLyrics = al_create_native_file_dialog(resourcePath.c_str(), "Set Text File", "*.txt;*.*", ALLEGRO_FILECHOOSER_SAVE);
}

Program::Program(int w, int h) {
	this->initAllegro(w, h);
		
	mainTimer = new Timer(FPS_TO_MS(30));
	al_register_event_source(queue, mainTimer->getEventSource());
	
	auto *path = al_get_standard_path(ALLEGRO_RESOURCES_PATH);
	resourcePath = al_path_cstr(path, ALLEGRO_NATIVE_PATH_SEP);
	al_destroy_path(path);
	
	this->initGui();

	size_t buffers = BUFFERS;
	unsigned samples = SAMPLES_PER_BUFFER;
	unsigned freq = STREAM_FREQUENCY;
	ALLEGRO_AUDIO_DEPTH depth = ALLEGRO_AUDIO_DEPTH_FLOAT32;
	ALLEGRO_CHANNEL_CONF ch = ALLEGRO_CHANNEL_CONF_1;
	
	synth = new Synth(buffers, samples, freq, depth, ch, 16);
	reader = new Interpreter;
	
	font_gui = al_load_ttf_font(std::string(resourcePath + std::string("data/DejaVuSans.ttf")).c_str(), 12, 0);
	if (!font_gui) {
		abort_example("Failed to load font\n");
	}
	
	synth->getVoice(6)->setWaveform(WAVEFORM_SAWTOOTH);
	synth->getVoice(2)->setWaveform(WAVEFORM_SINE);
	synth->getVoice(4)->setWaveform(WAVEFORM_SINE);
	synth->getVoice(0)->setWaveform(WAVEFORM_TRIANGLE);
	
	filenameP = nullptr;
	filenamePP = &filenameP;
	
	readerThread = al_create_thread(&interpreterThreadR, (void*) &programEventSource);
	audioThread = al_create_thread(&audioThreadR, nullptr);

}

Program::~Program() {
	delete mainTimer;
	al_destroy_font(font_gui);
	al_destroy_event_queue(queue);
	
	delete synth;
	delete reader;
}

void Program::run(const std::string& openFile) {
	mainTimer->start();
	
	al_start_thread(audioThread);
	al_start_thread(readerThread);
	
	if(openFile != "") {
		std::string endung = getExtension(openFile);
		if(endung == "mid" || endung == "midi") {
			std::cout << "[INFO] Starting with " << openFile << std::endl;
			this->addFile(openFile);
		} else if(endung == "m3u" || endung == "m3u8") {
			auto list = readPlaylist(openFile);
			std::cout << "[INFO] Playlist " << openFile << std::endl;
			for(int i = 0;i < list.size();i++) {
				addFile(list[i]);

 			}
		} else {
			al_show_native_message_box(nullptr, "Error", "Unrecognized File Type", std::string(std::string("Unrecognized File Extension: ") + openFile).c_str(), "I Guess", ALLEGRO_MESSAGEBOX_ERROR);
		}
	
	}
	
	while(running) {
		check();
		update();
		draw();
	}
	al_join_thread(audioThread, nullptr);
	al_join_thread(readerThread, nullptr);
	al_set_display_menu(display, nullptr);
}

void Program::check() {
	ALLEGRO_EVENT ev;
	while(al_get_next_event(queue, &ev)) {
		switch(ev.type) {
			case ALLEGRO_EVENT_DISPLAY_CLOSE:
				running = false;
				this->stop();
				break;
			case ALLEGRO_EVENT_TIMER:
				updateC++;
				break;
			case ALLEGRO_EVENT_MOUSE_AXES:
				mouseX = ev.mouse.x;
				mouseY = ev.mouse.y;
				break;
			case ALLEGRO_EVENT_MOUSE_BUTTON_DOWN:
				buttonState = true;
				break;
			case ALLEGRO_EVENT_DISPLAY_RESIZE:
				screenW = ev.display.width;
				screenH = ev.display.height;
				al_acknowledge_resize(display);
				break;
			case ALLEGRO_EVENT_MENU_CLICK:
				if(ev.user.data1 == FILE_OPEN_ID) {
					if(al_show_native_file_dialog(display, gui.fileDialog) && al_get_native_file_dialog_count(gui.fileDialog) > 0) {
						addFile(al_get_native_file_dialog_path(gui.fileDialog, 0));
						toLast();
					}
				} else if(ev.user.data1 == FILE_ADD_ID) {
					if(al_show_native_file_dialog(display, gui.fileDialog) && al_get_native_file_dialog_count(gui.fileDialog) > 0) {
						addFile(al_get_native_file_dialog_path(gui.fileDialog, 0));
					}
				} else if(ev.user.data1 == FILE_PLAY_ID) {
					this->play();
				} else if(ev.user.data1 == FILE_NEXT_ID) {
					this->next();
				} else if(ev.user.data1 == FILE_PREV_ID) {
					this->prev();
				} else if(ev.user.data1 == FILE_STOP_ID) {
					this->stop();
				} else if(ev.user.data1 == FILE_CLOSE_ID) {
					running = false;
					this->stop();
				} else if(ev.user.data1 == FILE_INFO_ID) {
					al_show_native_message_box(display, "About this excellent piece of software", "Expos Midi Player v. 0.5", "8-Bit Midi Player made in C++11 and Allegro with MidiFile and based on the Allegro \"Synth\" example program. Drum samples from http://freewavesamples.com", "Okay", 0);
				} else if(ev.user.data1 == FILE_EXPORT_LID) {
					if(reader->getFullLyrics() != "") {
						al_show_native_file_dialog(display, gui.exportLyrics);
						if(al_get_native_file_dialog_count(gui.exportLyrics) > 0) {
							std::string lyricfilename = al_get_native_file_dialog_path(gui.exportLyrics, 0);
							std::fstream lyric(lyricfilename, std::ios::out | std::ios::trunc);
							if(lyric) {
								lyric << reader->getFullLyrics();
								lyric.close();
							} else {
								al_show_native_message_box(nullptr, "Error", "Can't write to File", std::string(std::string("Can't write to the file") + lyricfilename).c_str(), "I Guess", ALLEGRO_MESSAGEBOX_ERROR);
							}
						}
					} else {
						al_show_native_message_box(nullptr, "Error", "No Lyrics Available", "The MIDI currently opened has no lyrics embedded!", "I Guess", ALLEGRO_MESSAGEBOX_ERROR);
					}
				} else if(ev.user.data1 >= FILE_CHANNEL_ID(0) && ev.user.data1 <= FILE_CHANNEL_ID(15)) {
					int channel = ev.user.data1 - FILE_CHANNEL_PRE;
					
					
					int index = 0;
					if(al_find_menu_item(gui.channels, ev.user.data1, nullptr, &index)) {
						std::cout << "   I: "<< index << std::endl;
						std::cout << "=> " << al_get_menu_item_flags(gui.channels, index) << std::endl;
						
						if(al_get_menu_item_flags(gui.channels, index) & ALLEGRO_MENU_ITEM_CHECKED) {
							std::cout << "Channel " << channel+1 << " (1-16) was demuted!" << std::endl;
							synth->getVoice(channel)->demute();
							//al_set_menu_item_flags(gui.settings, index, ALLEGRO_MENU_ITEM_CHECKBOX);
						} else {
							synth->getVoice(channel)->mute();
							std::cout << "Channel " << channel+1 << " (1-16) was muted!" << std::endl;
							//al_set_menu_item_flags(gui.settings, index, ALLEGRO_MENU_ITEM_CHECKED);
						}
					} else {
						std::cout << "Can't find it :(" << std::endl;
					}
				}
				break;
		}
	}
}

void Program::update() {
	while(updateC > 0) {
		redraw = true;
		updateC--;
	}
}

void Program::draw() {
	if(!redraw) return;
	
	al_clear_to_color(al_map_rgb(230, 230, 230));
		
	al_draw_textf(font_gui, al_map_rgb(0,0,0), 0, 0, 0, "%i/%i", reader->getTick(), reader->getLength());	
	
	al_draw_textf(font_gui, al_map_rgb(0,0,0), screenW / 2, 5, ALLEGRO_ALIGN_CENTRE, "\"%s\"", reader->getSongName().c_str());
	al_draw_textf(font_gui, al_map_rgb(0,0,0), screenW / 2, 5+ al_get_font_line_height(font_gui) , ALLEGRO_ALIGN_CENTRE, reader->getCopyright().c_str());
	al_draw_textf(font_gui, al_map_rgb(0,0,0), screenW / 2, screenH - al_get_font_line_height(font_gui) , ALLEGRO_ALIGN_CENTRE, reader->getLyrics().c_str());
	
	al_draw_textf(font_gui, al_map_rgb(0,0,0), 35, 350, 0, "Playlist: ");
	int startI = (reader->getPlaylistPos() > 3 ? reader->getPlaylistPos()-3 : 0);
	for(int i = startI;i < reader->getPlaylistSize();i++) {
		ALLEGRO_COLOR col = (reader->getPlaylistPos() == i ? al_map_rgb(255, 0, 0) : al_map_rgb(0,0,0));
		
		al_draw_textf(font_gui, col, 35, 350 + al_get_font_line_height(font_gui) * (i-startI+1), 0, "%i: %s", i+1, reader->getPlaylist(i).c_str());
	}
	
	/* Immer 4 Voices Nebeneinander */
	int vc = synth->getVoiceCount();
	int rows = vc / 4 + (vc % 4 ? 1 : 0);
	int cols = 4;
	int i = 0;
	for(int y = 0;y < rows;y++) {
		for(int x = 0;x < cols && i < vc;x++) {
			ALLEGRO_COLOR col = (synth->getVoice(i)->getFrequency() == 0.0 ? al_map_rgb(255,0,0) : al_map_rgb(0, 255, 0));
			
			al_draw_textf(font_gui, col, screenW / 8 + (screenW / 5 * x), 40 + (screenH / 7 * y) + al_get_font_line_height(font_gui) * 0, ALLEGRO_ALIGN_CENTRE, synth->getVoice(i)->getName().c_str());
			al_draw_textf(font_gui, col, screenW / 8 + (screenW / 5 * x), 40 + (screenH / 7 * y) + al_get_font_line_height(font_gui) * 1, ALLEGRO_ALIGN_CENTRE, "Waveform: %s", w2s(synth->getVoice(i)->getWaveform()));
			al_draw_textf(font_gui, col, screenW / 8 + (screenW / 5 * x), 40 + (screenH / 7 * y) + al_get_font_line_height(font_gui) * 2, ALLEGRO_ALIGN_CENTRE, "Frequency: %s", std::to_string(synth->getVoice(i)->getFrequency()).substr(0,6).c_str());
			
			
			if(mouseover(screenW / 8 + (screenW / 5 * x) - al_get_text_width(font_gui, "Dies ist ein Text"), 40 + (screenH / 7 * y) + al_get_font_line_height(font_gui) * 0, screenW / 8 + (screenW / 5 * (x)) + al_get_text_width(font_gui, "Dies ist ein Text"), 40 + (screenH / 7 * y) + al_get_font_line_height(font_gui) * 4)) {
				if(linksklick()) {
					int w = (int) synth->getVoice(i)->getWaveform();
					if(w != WAVEFORM_SAWTOOTH) {
						w++;
					} else {
						w = 0;
					}
					
					synth->getVoice(i)->setWaveform((Waveform) w);
				}
			}
			
			i++;
		}
	}	
	

	al_flip_display();
	redraw = false;
}
