#ifndef PROGRAM_H
#define PROGRAM_H
#include <allegro5/allegro_acodec.h>
#include <allegro5/allegro_native_dialog.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_ttf.h>
#include "synth.h"
#include "mfile/MidiFile.h"
#include "interpreter.h"
#include <bitset>


#define FILE_OPEN_ID    1
#define FILE_ADD_ID     2
#define FILE_PLAY_ID    3
#define FILE_STOP_ID    4
#define FILE_EXPORT_LID 5
#define FILE_CLOSE_ID   6
#define FILE_INFO_ID    7
#define FILE_NEXT_ID	8
#define FILE_PREV_ID	9


#define FILE_CHANNEL_PRE 8
#define FILE_CHANNEL_ID(x) (FILE_CHANNEL_PRE+x)


struct ProgramGui {
	ALLEGRO_MENU *menu;
	ALLEGRO_MENU *song_menu;
	ALLEGRO_MENU *file_menu;
	ALLEGRO_MENU *settings;
	ALLEGRO_MENU *channels;
	ALLEGRO_MENU *help;
	ALLEGRO_FILECHOOSER *fileDialog;
	ALLEGRO_FILECHOOSER *exportLyrics;
};

extern Interpreter *reader;

class Program {
	private:
		ALLEGRO_DISPLAY *display= nullptr;
		ALLEGRO_EVENT_QUEUE *queue= nullptr ;
		ALLEGRO_FONT *font_gui = nullptr;
		
		ProgramGui gui;
		
		Timer *mainTimer = nullptr;
		
		ALLEGRO_THREAD *readerThread= nullptr;
		ALLEGRO_THREAD *audioThread= nullptr ;
		
		ALLEGRO_EVENT_SOURCE programEventSource;
		
		void initAllegro(int, int);
		void initGui();
		
		const char *filename;
		const char **filenameP;
		const char ***filenamePP;
		
		bool running = true;
		int updateC = 0;
		bool redraw = true;
		
		void check();
		void update();
		void draw();
		
		void addFile(const std::string& tfilename) {
			// Setzt die Datei auf filename und spielt die ab
			ALLEGRO_EVENT event;
			event.user.type = EVENT_FILE_ADD;
			event.user.data1 = stringManager->push(tfilename);
			
			std::cout << "Event: " << tfilename << std::endl;
			
			al_emit_user_event(&programEventSource, &event, NULL);
		}
		
		void next() {
			// Setzt die Datei auf filename und spielt die ab
			ALLEGRO_EVENT event;
			event.user.type = EVENT_NEXT;
			al_emit_user_event(&programEventSource, &event, NULL);
		}
		
		void prev() {
			// Setzt die Datei auf filename und spielt die ab
			ALLEGRO_EVENT event;
			event.user.type = EVENT_PREV;
			al_emit_user_event(&programEventSource, &event, NULL);
		}
		
		void toLast() {
			// Setzt die Datei auf filename und spielt die ab
			ALLEGRO_EVENT event;
			event.user.type = EVENT_LAST;
			al_emit_user_event(&programEventSource, &event, NULL);
		}
		
		void play() {
			ALLEGRO_EVENT event;
			event.user.type = EVENT_PLAY;
			al_emit_user_event(&programEventSource, &event, NULL);
		}
		
		void stop() {
			ALLEGRO_EVENT event;
			event.user.type = EVENT_STOP;
			al_emit_user_event(&programEventSource, &event, NULL);
		}
	public:
		Program(int,int);
		~Program();
		
		void run(const std::string&);
};

#endif
