#include "synth.h"

bool saving = false;
ALLEGRO_FILE *save_fp = NULL;

Synth::Synth(size_t buffers, unsigned samples, unsigned freq, ALLEGRO_AUDIO_DEPTH depth, ALLEGRO_CHANNEL_CONF ch, int voiceCount) : delayTimer(0.001){
	if (!al_install_audio()) { throw(std::string("Could not init sound!\n")); }
	if(!al_init_acodec_addon()) { throw(std::string("Could not install codecs!\n")); }
	if (!al_reserve_samples(24)) { throw(std::string("Could not set up voice and mixer.\n")); }
	
	mixer = al_get_default_mixer();
	queue = al_create_event_queue();
	
	for(int i = 0;i < voiceCount;i++) {
		Voice *v = new Voice(i, buffers, samples, freq, depth, ch);
		
		v->setWaveform(WAVEFORM_SQUARE);
		v->addToMixer(mixer);
		if(v->getWaveform() == WAVEFORM_SAWTOOTH) {
			al_set_audio_stream_gain(v->getStream(), 0.3);
		} else {
			al_set_audio_stream_gain(v->getStream(), 0.5);
		}
		al_register_event_source(queue, v->getEventSource());
		
		voices.push_back(v);
	}
	
	for(int i = 35;i <= 58;i++) {
		std::string fn = resourcePath + std::string("data/drums/") + std::to_string(i) + std::string(".wav");
		auto *s = al_load_sample(fn.c_str());
		if(s == nullptr) {
			std::cerr << "[ERR.] Can't open Drum Sample " << fn << std::endl;
		} else {
			drumsamples[i] = s;
		}
		
	}
	
	al_register_event_source(queue, delayTimer.getEventSource());
	
	al_set_mixer_postprocess_callback(mixer, mixer_pp_callback, mixer);
	delayTimer.start();
}

Synth::~Synth() {
	for(int i = 0;i < voices.size();i++) {
		delete voices[i];
	}
	al_destroy_event_queue(queue);
}

void Synth::reactEvent(int trackID, const MidiEvent& event) {
	if(event.isNoteOn()) {
		short channel = event.getChannelNibble();
		if(channel != 9) {
			this->getVoice(channel)->setFrequency(noteToFreq(event[1]));
		} else {
			auto sample = event[1];
			if(!this->getVoice(9)->isMuted() && drumsamples.find(sample) != drumsamples.end()) {
				delayedSamples.push_back({sample, delay});
			}
		}
		
	} else if(event.isNoteOff()) {
		short channel = event.getChannelNibble();
		this->getVoice(channel)->setFrequency(0.0);
	}
}

void Synth::generate(int voiceNum, int samples) {
	if(voiceNum == 9) return;
	Voice *voice = this->getVoice(voiceNum);
	
	void *buf;
	float gain;
	float pan;


	buf = al_get_audio_stream_fragment(voice->getStream());
	if (!buf) return; //Kann passieren
	

	Waveform type = voice->getWaveform();
	float frequency = voice->getFrequency();
	float phase = voice->getPhase();

	generate_wave(type, (float*)buf, samples, voice->getT(), frequency, phase);

	voice->addT(dt * samples);
	/*
	if (group->get_gain_if_changed(&gain)) {
		al_set_audio_stream_gain(stream, gain);
	}
if (group->get_pan_if_changed(&pan)) {
		al_set_audio_stream_pan(stream, pan);
	}
*/

	if (!al_set_audio_stream_fragment(voice->getStream(), buf)) {
		std::cout << "Error setting Stream Fragment!" << std::endl;
	}
}


void Synth::check() {
	ALLEGRO_EVENT ev;
	while(al_get_next_event(queue, &ev)) {
		switch(ev.type) {
			case ALLEGRO_EVENT_AUDIO_STREAM_FRAGMENT:
			reactFragment((ALLEGRO_AUDIO_STREAM *) ev.any.source);
			break;
			case ALLEGRO_EVENT_TIMER:
				for(int i = 0;i < delayedSamples.size();i++) {
					delayedSamples[i].delay -= 0.001;
					if(delayedSamples[i].delay <= 0) {
						al_play_sample(drumsamples[delayedSamples[i].sample], 0.5, 0.5, 1.0, ALLEGRO_PLAYMODE_ONCE, nullptr);
						delayedSamples.erase(delayedSamples.begin() + i);
					}
				}
				break;
		}
	}
}
 Synth *synth;
