#ifndef SYNTH_H
#define SYNTH_H
#include <stdio.h>
#include <math.h>
#include <vector>
#include "allegro5/allegro_audio.h"
#include <allegro5/allegro_acodec.h>
#include "voice.h"
#include "timer.h"

extern bool saving;
extern ALLEGRO_FILE *save_fp;

static void mixer_pp_callback(void *buf, unsigned int samples, void *userdata) { ALLEGRO_MIXER *mixer = (ALLEGRO_MIXER *)userdata;
	int nch;
	int sample_size;

	if (!saving)
		return;

	switch (al_get_mixer_channels(mixer)) {
		case ALLEGRO_CHANNEL_CONF_1:
			nch = 1;
			break;
		case ALLEGRO_CHANNEL_CONF_2:
			nch = 2;
			break;
		default:
			/* Not supported. */
			return;
	}

	sample_size = al_get_audio_depth_size(al_get_mixer_depth(mixer));
	al_fwrite(save_fp, buf, nch * samples * sample_size);
}

struct SampleDelay {
	unsigned char sample;
	double delay;
};

class Synth : public MidiEventListener {
	private:
		ALLEGRO_MIXER *mixer;
		ALLEGRO_EVENT_QUEUE *queue;
		Timer delayTimer;
		
		std::vector<SampleDelay> delayedSamples;
		
		std::vector<Voice*> voices;
		
		void reactFragment(ALLEGRO_AUDIO_STREAM *stream) {
			void *buf;
			float gain;
			float pan;
		
			int voice = this->getVoiceByStream(stream);
			if(voice != -1) {
				this->generate(voice, SAMPLES_PER_BUFFER);	
			} else {
				std::cout << "gdlpfkgdfojgpdfokgpodfkg" << std::endl;
			}
		
		}
	public://Delay Timer
		Synth(size_t buffers, unsigned samples, unsigned freq, ALLEGRO_AUDIO_DEPTH depth, ALLEGRO_CHANNEL_CONF ch, int voiceCount);
		~Synth();
		
		void panic() {
			for(int i = 0;i < voices.size();i++) {
				voices[i]->setFrequency(0.0);
			}
		}
		
		void reactEvent(int trackID, const MidiEvent& event);
		void generate(int voiceNum, int samples);
		void check();
		
		int getVoiceCount() {
			return voices.size();
		}
		
		Voice *getVoice(int n) {
			return voices[n];
		}
		
		int getVoiceByStream(ALLEGRO_AUDIO_STREAM *stream) {
			for(int i = 0;i < voices.size();i++) {
				if(stream == voices[i]->getStream()) return i;
			}
			return -1;
		}
		
		ALLEGRO_AUDIO_STREAM *getStream(int n) {
			return voices[n]->getStream();
		}
};

extern Synth *synth;


#endif
