#ifndef TIMER_H
#define TIMER_H
#include "allegro5/allegro.h"
#include "mfile/MidiFile.h"

class Timer {
	private:
		ALLEGRO_TIMER *timer;
	public:
		Timer(double speed) {
			timer = al_create_timer(speed);
		}
		
		~Timer() {
			al_destroy_timer(timer);
		}
		
		void start() {
			al_start_timer(timer);
		}
		
		void resume() {
			al_resume_timer(timer);
		}
		
		void stop() {
			if(!this->active()) return;
			al_stop_timer(timer);
		}
		
		int64_t getCount() {
			return al_get_timer_count(timer);
		}
		
		void setCount(int64_t count) {
			al_set_timer_count(timer, count);
		}
		
		void addCount(int64_t value) {
			al_add_timer_count(timer, value);
		}
		
		double getSpeed() {
			return al_get_timer_speed(timer);
		}
		
		void setSpeed(double speed) {
			al_set_timer_speed(timer, speed);
		}
		
		ALLEGRO_EVENT_SOURCE *getEventSource() {
			return al_get_timer_event_source(timer);
		}
		
		bool active() {
			return al_get_timer_started(timer);
		}
};

double mtempoToSpeed(unsigned int mSecQuarter, int ticksPerQuarter);


inline int mtempoToBPM(unsigned int mSecQuarter) {
	/* Converts microseconds per quarter to BPM, see mtempoToSpeed for helpful links */
	unsigned long long tempoThing = 60000000 / mSecQuarter;
	return (unsigned int) tempoThing;
}

unsigned int decodeTempo(const MidiEvent& tempoChange);

#endif
