#ifndef VOICE_H
#define VOICE_H
#include "allegro5/allegro_audio.h"
#include <vector>
#include <iostream>
#include "base.h"

class Voice {
	private:
		std::string name;
		int myID = 0;
		
		ALLEGRO_AUDIO_STREAM *stream;
		Waveform type = WAVEFORM_NONE;
		float phase = 0;
		float t = 0;
		float myF = 0;
		
		bool deactivated = false;
	public:
		Voice(int tid, size_t buffers, unsigned samples, unsigned freq, ALLEGRO_AUDIO_DEPTH depth, ALLEGRO_CHANNEL_CONF ch) : myID(tid) {
			stream = al_create_audio_stream(buffers, samples, freq, depth, ch);
			name = std::string("Channel ") + std::to_string(myID+1);
		}
		
		~Voice() {
			al_destroy_audio_stream(stream);
		}
		
		void mute() {
			deactivated = true;
		}
		
		void demute(){
			deactivated = false;
		}
		
		bool isMuted() {
			return deactivated;
		}
		
		void setName(const std::string& newName) {
			if(newName == "") name = std::string("Channel ") + std::to_string(myID+1);
			name = newName;
		}
		
		const std::string& getName() {
			return name;
		}
		
		void setWaveform(Waveform form) {
			type = form;
		}
		
		Waveform getWaveform() {
			return type;
		}
		
		float getFrequency() {
			if(deactivated) return 0.0;
			return myF;
		}
		
		void setFrequency(float f) {
			
			myF = f;
		}
		
		float getPhase() {
			return phase;
		}
		
		float getT() {
			return t;
		}
		
		void addT(float val) {
			t += val;
		}
		
		void addToMixer(ALLEGRO_MIXER *mixer) {
			al_attach_audio_stream_to_mixer(stream, mixer);
		}

		ALLEGRO_EVENT_SOURCE *getEventSource() {
			return al_get_audio_stream_event_source(stream);
		}
		
		ALLEGRO_AUDIO_STREAM *getStream() {
			return stream;
		}
};

#endif
