#include "lib/program.h"
#include <exception>

/* For an explanation of the MIDIFile (everything in the folder mfile/) go to 
	http://midifile.sapp.org/
	
	A basic understanding of the MIDI File Format is probably neccessary to understand the code
	fully, so a short introduction to everything worth knowing can be found at http://www.ccarh.org/courses/253/handout/smf/

	generation.h	-	Functions for generating the different wave forms
	base.h			-	Basic Routines and classes for the rest of the program
	timer.h			-	Timer Class, dependent on Allegro and needed by the Midi Interpreter for timing
	
	Audio-Core:
	voice.h			-	Voice Class (represents a MIDI Channel, currently no polyphony)
	synth.h			-	Synth Class (recieves the MIDI events & sets the Voices accordingly and fills the audio streams of the voices)
	
	Interpreter:
	track.h			-	Track (= a MIDI track, that contains Events, that the Track sends to registered MidiEventListeners (see base.h for that class))
	interpreter.h	-	Manager for all MIDI-File stuff, keeps track of the playlist and the currently opened file and most importantly the tracks.
	
	Program:
	program.h		-	The Main Application, Runs the Audio Core (synth->check()) and the Interpreter (interpreter->check()) in a loop in a seperate Thread and sends the file open, play etc. Events to the Interpreter
	
	
	Look at readme.txt for more
*/
		

int main(int argc, char *argv[]) {
	try {
		stringManager = new StringManager;
		
		std::string filename = "";
		if(argc > 1) filename = argv[1];
		
		Program *myProgram = new Program(800, 600);
		
		myProgram->run(filename);
		
		delete myProgram;
	} catch(std::string& str) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured:", str.c_str(), "Oh no!", ALLEGRO_MESSAGEBOX_ERROR);
	} catch(std::exception& e) {
		al_show_native_message_box(nullptr, "Error", "An exception has occured:", e.what(), "Oh bugger!", ALLEGRO_MESSAGEBOX_ERROR);
	}
	
	return 0;
}

/* vim: set sts=3 sw=3 et: */
